﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;
using ImageProcessor;

namespace Image_Processor
{
	public partial class Form1 : Form
	{
		private int pic = 0;
		private List<CustomImageProcessor> images = new List<CustomImageProcessor>();
		private CustomImageProcessor currentImage; // Used only if "Keep all images in memory" turned off
		private ProcessSequence processSequence = new ProcessSequence();
		private bool loadAllImages = false;

        public Form1()
        {
            InitializeComponent();
        }

		private void SelectFilesBtn_Click(object sender, EventArgs e)
		{
			if (OpenFileDialog1.ShowDialog() == DialogResult.OK)
			{
				PictureBoxMain.LoadAsync(OpenFileDialog1.FileName);
				SelectDirLabel.Text = Path.GetDirectoryName(OpenFileDialog1.FileName);
				if (loadAllImages)
				{
					foreach (string filePath in OpenFileDialog1.FileNames)
					{
						images.Add(new CustomImageProcessor(filePath));
					}
				}
				UpdatePictureBox();
				PreviewImageTip.Active = true;
			}
		}

		private void KeepImagesInMemCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			loadAllImages = KeepImagesInMemCheckBox.Checked;
			if (loadAllImages && PictureBoxMain.Image != null)
			{
				foreach (string filePath in OpenFileDialog1.FileNames)
				{
					images.Add(new CustomImageProcessor(filePath));
				}
			}
			else
			{
				for (int i = images.Count() - 1; i >= 0; i--)
				{
					images[i].Dispose();
				}
				images = new List<CustomImageProcessor>();
				GC.Collect();
			}

			if (PictureBoxMain.Image != null)
			{
				UpdatePictureBox();
			}
		}

		private void OpenFileDialog1_FileOk(object sender, CancelEventArgs e)
		{
			pic = 0;
		}

		// --------------------BATCH_PROCESSING---------------------
		private void SaveFilesBtn_Click(object sender, EventArgs e)
		{
			ProgressBar.Value = 0;
			ProgressBar.Maximum = OpenFileDialog1.FileNames.Length * 2;
			ProgressBar.Visible = true;
			BackgroundWorker1.RunWorkerAsync();
		}

		private void BackgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
		{
			BackgroundWorker worker = sender as BackgroundWorker;

			if (loadAllImages)
			{
				Parallel.For(0, images.Count, async i =>
				{
					worker.ReportProgress(2 * i);
					images[i].ProcessImage(processSequence);
					if (FilenameAppendNumCheckBox.Checked)
						await images[i].SaveImage("_" + i, SelectDirLabel.Text + "\\");
					if (FilenameKeepNameCheckBox.Checked)
						await images[i].SaveImage("", SelectDirLabel.Text + "\\");
					if (FilenameAppendStrCheckBox.Checked)
						await images[i].SaveImage(FilenameTextBox.Text, SelectDirLabel.Text + "\\");
					try { worker.ReportProgress(2 * i + 1); }
					catch { }
				});
			}
			else
			{
				Parallel.For(0, OpenFileDialog1.FileNames.Length, async i =>
				{
					worker.ReportProgress(2 * i);
					CustomImageProcessor image = new CustomImageProcessor(OpenFileDialog1.FileNames[i]);
					image.ProcessImage(processSequence);
					if (FilenameAppendNumCheckBox.Checked)
						await image.SaveImage("_" + i, SelectDirLabel.Text + "\\");
					if (FilenameKeepNameCheckBox.Checked)
						await image.SaveImage("", SelectDirLabel.Text + "\\");
					if (FilenameAppendStrCheckBox.Checked)
						await image.SaveImage(FilenameTextBox.Text, SelectDirLabel.Text + "\\");
					try { worker.ReportProgress(2 * i + 1); }
					catch { }
				});
			}
			GC.Collect();
		}

		private void BackgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			ProgressBar.PerformStep();
		}

		private void BackgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			ProgressBar.Value = ProgressBar.Maximum;
			ProgressBar.Visible = false;
		}
		// ---------------------------------------------------------

		private void UpdatePictureBox()
		{
			if (PictureBoxMain.Image != null)
			{
				if (loadAllImages)
				{
					images[pic].ProcessImage(processSequence);
					PictureBoxMain.Image = images[pic].GetImage();
				}
				else
				{
					currentImage = new CustomImageProcessor(OpenFileDialog1.FileNames[pic]);
					currentImage.ProcessImage(processSequence);
					PictureBoxMain.Image = currentImage.GetImage();
				}
			}
		}

		private void PrevImageBtn_Click(object sender, EventArgs e)
		{
			int original = pic;
			if (--pic < 0)
			{
				pic = OpenFileDialog1.FileNames.Length - 1;
			}
			if (pic != original)
			{
				UpdatePictureBox();
			}
		}

		private void NextImageBtn_Click(object sender, EventArgs e)
		{
			int original = pic;
			if (++pic >= OpenFileDialog1.FileNames.Length)
			{
				pic = 0;
			}
			if (pic != original)
			{
				UpdatePictureBox();
			}
		}

		private void PictureBox1_MouseDown(object sender, MouseEventArgs e)
		{
			if (PreviewImageTip.Active)
			{
				PictureBoxMain.LoadAsync(OpenFileDialog1.FileNames[pic]);
			}
		}

		private void PictureBox1_MouseUp(object sender, MouseEventArgs e)
		{
			if (PreviewImageTip.Active)
			{
				if (loadAllImages)
				{
					PictureBoxMain.Image = images[pic].GetImage();
				}
				else
				{
					PictureBoxMain.Image = currentImage.GetImage();
				}
			}
		}

		private void ModifySequence(ProcessMethod.Name name, int value) {
			ProcessMethod process = new ProcessMethod(name, value);
			processSequence.ModifyProcess(process);
			UpdatePictureBox();
		}

		private void ModifySequence(ProcessMethod process)
		{
			processSequence.ModifyProcess(process);
			UpdatePictureBox();
		}

		private Color SelectColor()
		{
			ColorDialog SelectColorDialog = new ColorDialog();
			SelectColorDialog.FullOpen = true;
			if (SelectColorDialog.ShowDialog() == DialogResult.OK)
			{
				return SelectColorDialog.Color;
			}
			return new Color();
		}

		// ---------------------------------------------------------
		// ------------------------CONTROLS-------------------------
		// ---------------------------------------------------------

		private void FlowLayoutPanel1_Resize(object sender, EventArgs e)
		{
			if (FlowLayoutPanel1.VerticalScroll.Visible)
			{
				TableLayoutPanel1.ColumnStyles[0].Width = 352;
			}
			else
			{
				TableLayoutPanel1.ColumnStyles[0].Width = 331;
			}
		}

		// -----------------------BRIGHTNESS------------------------
		private void BrightnessCheckbox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Brightness, BrightnessBar.Value);
			processSequence.AddOrRemove(process, BrightnessCheckBox.Checked);
			UpdatePictureBox();
		}

		private void BrightnessBar_Scroll(object sender, EventArgs e)
		{
			BrightnessNumeric.Value = BrightnessBar.Value;
		}

		private void BrightnessNumeric_ValueChanged(object sender, EventArgs e)
		{
			BrightnessBar.Value = (int) BrightnessNumeric.Value;
			if (BrightnessCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Brightness, BrightnessBar.Value);
			}
		}

		// ------------------------OPACITY--------------------------
		private void OpacityCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Alpha, OpacityBar.Value);
			processSequence.AddOrRemove(process, OpacityCheckBox.Checked);
			UpdatePictureBox();
		}

		private void OpacityBar_Scroll(object sender, EventArgs e)
		{
			OpacityNumeric.Value = OpacityBar.Value;
		}

		private void OpacityNumeric_ValueChanged(object sender, EventArgs e)
		{
			OpacityBar.Value = (int) OpacityNumeric.Value;
			if (OpacityCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Alpha, OpacityBar.Value);
			}
		}

		// -----------------------SATURATION------------------------
		private void SaturationCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Saturation, SaturationBar.Value);
			processSequence.AddOrRemove(process, SaturationCheckBox.Checked);
			UpdatePictureBox();
		}

		private void SaturationBar_Scroll(object sender, EventArgs e)
		{
			SaturationNumeric.Value = SaturationBar.Value;
		}

		private void SaturationNumeric_ValueChanged(object sender, EventArgs e)
		{
			SaturationBar.Value = (int) SaturationNumeric.Value;
			if (SaturationCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Saturation, SaturationBar.Value);
			}
		}

		// -----------------------CONTRAST--------------------------
		private void ContrastCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Contrast, ContrastBar.Value);
			processSequence.AddOrRemove(process, ContrastCheckBox.Checked);
			UpdatePictureBox();
		}

		private void ContrastBar_Scroll(object sender, EventArgs e)
		{
			ContrastNumeric.Value = ContrastBar.Value;
		}

		private void ContrastNumeric_ValueChanged(object sender, EventArgs e)
		{
			ContrastBar.Value = (int) ContrastNumeric.Value;
			if (ContrastCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Contrast, ContrastBar.Value);
			}
		}

		// ---------------------EDGE DETECTION----------------------
		private void DetectEdgesCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.EdgeDetection);
			int index = EdgeDetectionComboBox.SelectedIndex;
			index = GreyscaleCheckBox.Checked ? index + 10 : index;
			process.value = index;
			bool add = DetectEdgesCheckBox.Checked && EdgeDetectionComboBox.SelectedIndex >= 0;
			processSequence.AddOrRemove(process, add);
			UpdatePictureBox();
		}

		private void EdgeDetectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (DetectEdgesCheckBox.Checked && EdgeDetectionComboBox.SelectedIndex >= 0)
			{
				int index = EdgeDetectionComboBox.SelectedIndex;
				index = GreyscaleCheckBox.Checked ? index + 10 : index;
				ModifySequence(ProcessMethod.Name.EdgeDetection, index);
			}
		}

		private void GreyscaleCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			EdgeDetectionComboBox_SelectedIndexChanged(sender, e);
		}

		// -------------------------FILTER--------------------------
		private void FilterCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Filter, FilterComboBox.SelectedIndex);
			processSequence.AddOrRemove(process, FilterCheckBox.Checked && FilterComboBox.SelectedIndex >= 0);
			UpdatePictureBox();
		}

		private void FilterComboBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (FilterCheckBox.Checked && FilterComboBox.SelectedIndex >= 0)
			{
				ModifySequence(ProcessMethod.Name.Filter, FilterComboBox.SelectedIndex);
			}
		}

		// --------------------------TINT---------------------------
		private void TintCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Tint, 0, TintColorLabel.BackColor);
			processSequence.AddOrRemove(process, TintCheckBox.Checked);
			UpdatePictureBox();
		}

		private void TintSelectColorBtn_Click(object sender, EventArgs e)
		{
			TintColorLabel.BackColor = SelectColor();
			if (TintCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Tint, 0, TintColorLabel.BackColor);
				ModifySequence(process);
			}
		}

		// ---------------------------HUE---------------------------
		private void HueCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Hue);
			process.value = HueRotateCheckBox.Checked ? 0 : 1000;
			process.value += HueBar.Value;
			processSequence.AddOrRemove(process, HueCheckBox.Checked);
			UpdatePictureBox();
		}

		private void HueRotateCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			HueAdjustCheckBox.Checked = !HueRotateCheckBox.Checked;
			if (HueCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Hue, HueBar.Value);
				ModifySequence(process);
			}
		}

		private void HueAdjustCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			HueRotateCheckBox.Checked = !HueAdjustCheckBox.Checked;
			if (HueCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Hue, HueBar.Value + 1000);
				ModifySequence(process);
			}
		}

		private void HueBar_Scroll(object sender, EventArgs e)
		{
			HueNumeric.Value = HueBar.Value;
		}

		private void HueNumeric_ValueChanged(object sender, EventArgs e)
		{
			HueBar.Value = (int) HueNumeric.Value;
			if (HueCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Hue, HueBar.Value);
			}
		}

		// --------------------------BLUR---------------------------
		private void BlurCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Blur);
			processSequence.AddOrRemove(process, BlurCheckBox.Checked);
			UpdatePictureBox();
		}

		private void BlurBar_Scroll(object sender, EventArgs e)
		{
			BlurNumeric.Value = BlurBar.Value;
		}

		private void BlurNumeric_ValueChanged(object sender, EventArgs e)
		{
			BlurBar.Value = (int) BlurNumeric.Value;
			if (BlurCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Blur, BlurBar.Value);
			}
		}

		// -------------------------SHARPEN-------------------------
		private void SharpenCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Sharpen);
			processSequence.AddOrRemove(process, SharpenCheckBox.Checked);
			UpdatePictureBox();
		}

		private void SharpenBar_Scroll(object sender, EventArgs e)
		{
			SharpenNumeric.Value = SharpenBar.Value;
		}

		private void SharpenNumeric_ValueChanged(object sender, EventArgs e)
		{
			SharpenBar.Value = (int) SharpenNumeric.Value;
			if (SharpenCheckBox.Checked)
			{
				ModifySequence(ProcessMethod.Name.Sharpen, SharpenBar.Value);
			}
		}

		// ---------------------REPLACE_COLOR-----------------------
		private void ReplaceColorCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.ReplaceColor);
			processSequence.AddOrRemove(process, ReplaceColorCheckBox.Checked);
			UpdatePictureBox();
		}

		private void ReplaceColorBar_Scroll(object sender, EventArgs e)
		{
			ReplaceColorNumeric.Value = ReplaceColorBar.Value;
		}

		private void ReplaceColorNumeric_ValueChanged(object sender, EventArgs e)
		{
			ReplaceColorBar.Value = (int) ReplaceColorNumeric.Value;
			if (ReplaceColorCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.ReplaceColor)
				{
					value = ReplaceColorBar.Value,
					color = ReplaceColorFromColor.BackColor,
					secondaryColor = ReplaceColorToColor.BackColor
				};
				ModifySequence(process);
			}
		}

		private void ReplaceColorFromBtn_Click(object sender, EventArgs e)
		{
			ReplaceColorFromColor.BackColor = SelectColor();
			if (ReplaceColorCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.ReplaceColor)
				{
					value = ReplaceColorBar.Value,
					color = ReplaceColorFromColor.BackColor,
					secondaryColor = ReplaceColorToColor.BackColor
				};
				ModifySequence(process);
			}
		}

		private void ReplaceColorToBtn_Click(object sender, EventArgs e)
		{
			ReplaceColorToColor.BackColor = SelectColor();
			if (ReplaceColorCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.ReplaceColor)
				{
					value = ReplaceColorBar.Value,
					color = ReplaceColorFromColor.BackColor,
					secondaryColor = ReplaceColorToColor.BackColor
				};
				ModifySequence(process);
			}
		}

		// -----------------------VIGNETTE--------------------------
		private void VignetteCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Vignette, 0, VignetteColorLabel.BackColor);
			processSequence.AddOrRemove(process, VignetteCheckBox.Checked);
			UpdatePictureBox();
		}

		private void VignetteColorBtn_Click(object sender, EventArgs e)
		{
			VignetteColorLabel.BackColor = SelectColor();
			if (VignetteCheckBox.Checked)
			{
				ProcessMethod process = new ProcessMethod(ProcessMethod.Name.Vignette, 0, VignetteColorLabel.BackColor);
				ModifySequence(process);
			}
		}

		// ---------------------SAVE_CONTROLS-----------------------
		private void DirDialogBtn_Click(object sender, EventArgs e)
		{
			if (FolderBrowserDialog1.ShowDialog() == DialogResult.OK)
			{
				SelectDirLabel.Text = FolderBrowserDialog1.SelectedPath;
			}
		}

		// ------------------FILENAME_FORMATTING--------------------
		private void SelectFormattingBox(int index)
		{
			CheckBox[] boxes =
			{
				FilenameAppendNumCheckBox,
				FilenameKeepNameCheckBox,
				FilenameAppendStrCheckBox
			};
			if (boxes[index].Checked && Convert.ToInt32(FilenameFormatLabel.Tag) != index)
			{
				int temp = Convert.ToInt32(FilenameFormatLabel.Tag);
				FilenameFormatLabel.Tag = index;
				boxes[temp].Checked = false;
			}
			else if (Convert.ToInt32(FilenameFormatLabel.Tag) == index)
			{
				boxes[index].Checked = true;
			}
		}

		private void FilenameAppendNumCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			SelectFormattingBox(0);
		}

		private void FilenameKeepNameCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			SelectFormattingBox(1);
		}

		private void FilenameAppendStrCheckBox_CheckedChanged(object sender, EventArgs e)
		{
			SelectFormattingBox(2);
		}
	}
}
