﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ImageProcessor;
using ImageProcessor.Imaging.Filters.EdgeDetection;
using ImageProcessor.Imaging.Filters.Photo;

namespace Image_Processor
{
	struct ProcessMethod
	{
		public enum Name
		{
			Brightness,
			Alpha,
			Saturation,
			Contrast,
			EdgeDetection,
			Filter,
			Tint,
			Hue,
			Blur,
			Sharpen,
			ReplaceColor,
			Vignette
		}
		public Name name;
		public int value;
		public Color color;
		public Color secondaryColor;

		public ProcessMethod(Name name, int value=0, Color color=new Color())
		{
			this.name = name;
			this.value = value;
			this.color = color;
			secondaryColor = color;
		}

		public ImageProcessor.Imaging.Filters.EdgeDetection.IEdgeFilter GetEdgeDetectionFilter()
		{
			if (value >= 10)
			{
				value -= 10;
			}
			ImageProcessor.Imaging.Filters.EdgeDetection.IEdgeFilter[] filters =
			{
				new KayyaliEdgeFilter(),
				new KirschEdgeFilter(),
				new Laplacian3X3EdgeFilter(),
				new Laplacian5X5EdgeFilter(),
				new LaplacianOfGaussianEdgeFilter(),
				new PrewittEdgeFilter(),
				new RobertsCrossEdgeFilter(),
				new ScharrEdgeFilter(),
				new SobelEdgeFilter()
			};
			return filters[value];
		}

		public IMatrixFilter GetMatrixFilter()
		{
			IMatrixFilter[] filters =
			{
				MatrixFilters.BlackWhite,
				MatrixFilters.Comic,
				MatrixFilters.Gotham,
				MatrixFilters.GreyScale,
				MatrixFilters.HiSatch,
				MatrixFilters.Invert,
				MatrixFilters.Lomograph,
				MatrixFilters.LoSatch,
				MatrixFilters.Polaroid,
				MatrixFilters.Sepia
			};
			return filters[value];
		}
	}
}
